//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include "App_Programs.h"
#include "Moa_AutoPointers.h"
#include "Moa_Files.h"
#include "Moa_ShellFiles.h"
#include "Moa_ShellLinks.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_programs {
//---------------------------------------------------------------------------
void __fastcall TShortcutParameters::Initialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TShortcutParameters::Initialize(const AnsiString& AFile)
{
	try {
		TAutoPointer<TShellLink> LShellLink = new TShellLink();
		LShellLink->LoadFromFile(AFile);
		FTarget = LShellLink->Target;
		FWorkingDirectory = LShellLink->WorkingDirectory;
		FShowCommand = LShellLink->ShowCommand;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TShortcutParameters::Uninitialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TShortcutParameters::TShortcutParameters() :
	TObject(),
	FTarget(),
	FWorkingDirectory(),
	FShowCommand(0)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TShortcutParameters::TShortcutParameters(const AnsiString& AFile) :
	TObject(),
	FTarget(),
	FWorkingDirectory(),
	FShowCommand()
{
	Initialize(AFile);
}
//---------------------------------------------------------------------------
__fastcall TShortcutParameters::~TShortcutParameters()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TShortcutParameters::Assign(TShortcutParameters* AOther)
{
	if(AOther == this) {
		return;
	}
	FTarget = AOther->FTarget;
	FWorkingDirectory = AOther->FWorkingDirectory;
	FShowCommand = AOther->FShowCommand;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
AnsiString __fastcall TProgramEntry::EnsureValidPath(const AnsiString& APath)
{
	AnsiString LPath = APath;
	try {
		TPathUtilities::AssureAbsolute(LPath);
		LPath = ExpandFileName(LPath);
		LPath = TFileUtilities::GetLongPath(LPath);
	}
	catch(EFileError& LException) {
		throw EProgramError("The program cannot accept the path.");
	}
	return LPath;
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Initialize()
{
	try {
		FIcon = new TIcon();
		if(Default) {
			TShellFileUtilities::GetIcon(
				TShellFileUtilities::FindIconByExtension("exe", isSmall),
				FIcon,
				isSmall);
			FParameters = new TShortcutParameters();
		}
		else {
			TShellFileUtilities::GetIcon(
				TShellFileUtilities::FindIconByPath(FFile, isSmall),
				FIcon,
				isSmall);
			FParameters = new TShortcutParameters(FFile);
		}
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Uninitialize()
{
	delete FIcon;
	FIcon = NULL;
}
//---------------------------------------------------------------------------
__fastcall TProgramEntry::TProgramEntry() :
	TObject(),
	FIcon(NULL),
	FFile(),
	FName("Default - depends on file association"),
	FParameters(NULL),
	FDefault(true)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TProgramEntry::TProgramEntry(const AnsiString& AFile) :
	TObject(),
	FIcon(NULL),
	FFile(EnsureValidPath(AFile)),
	FName(ChangeFileExt(ExtractFileName(FFile), AnsiString())),
	FParameters(NULL),
	FDefault(false)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TProgramEntry::~TProgramEntry()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Start()
{
	if(Default) {
		throw ECommandError("The program cannot start the program.");
	}
	AnsiString LFile = File;
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
	LExecuteInfo.hwnd = Application->Handle;
	LExecuteInfo.lpFile = LFile.c_str();
	LExecuteInfo.nShow = SW_SHOWNORMAL;
	if(!::ShellExecuteEx(&LExecuteInfo)) {
		throw ECommandError("The program could not start the program.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Explore()
{
	if(Default) {
		throw ECommandError("The program cannot show the directory.");
	}
	AnsiString LDirectory = ExtractFileDir(Parameters->Target);
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
	LExecuteInfo.hwnd = Application->Handle;
	LExecuteInfo.lpVerb = "explore";
	LExecuteInfo.lpFile = LDirectory.c_str();
	LExecuteInfo.nShow = SW_SHOWNORMAL;
	if(!::ShellExecuteEx(&LExecuteInfo)) {
		throw ECommandError("The program could not show the directory.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::ShowProperties()
{
	if(Default) {
		throw ECommandError("The program cannot show the properties.");
	}
	AnsiString LFile = File;
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI | SEE_MASK_INVOKEIDLIST;
	LExecuteInfo.hwnd = Application->Handle;
	LExecuteInfo.lpVerb = "properties";
	LExecuteInfo.lpFile = LFile.c_str();
	LExecuteInfo.nShow = SW_SHOWNORMAL;
	if(!::ShellExecuteEx(&LExecuteInfo)) {
		throw ECommandError("The program could not show the properties.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramEntry::Assign(TProgramEntry* AOther)
{
	if(AOther == this) {
		return;
	}
	FIcon->Assign(AOther->FIcon);
	FName = AOther->FName;
	FFile = AOther->FFile;
	FParameters->Assign(AOther->FParameters);
	FDefault = AOther->FDefault;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
const AnsiString TProgramModel::ProgramsDirectory = ExtractFilePath(Application->ExeName) + "Programs";
const AnsiString TProgramModel::ReadmeFile = IncludeTrailingPathDelimiter(ProgramsDirectory) + "Readme.txt";
//---------------------------------------------------------------------------
void __fastcall TProgramModel::EnsureProgramsDirectoryExists()
{
	if(DirectoryExists(ProgramsDirectory)) {
		return;
	}
	try {
		TDirectoryUtilities::FullCreate(ProgramsDirectory);
	}
	catch(EFileError& LException) {
		throw ECommandError("The program could not create the \"Programs\" directory.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramModel::EnsureReadmeFileExists()
{
	if(FileExists(ReadmeFile)) {
		return;
	}
	try {
		TAutoPointer<TStrings> LReadme = new TStringList();
		LReadme->Add("To register a program, place a shortcut file of the program in this directory.");
		LReadme->SaveToFile(ReadmeFile);
	}
	catch(EStreamError& LException) {
		throw ECommandError("The program could not create the readme file.");
	}
}
//---------------------------------------------------------------------------
int __fastcall TProgramModel::ReloadCompare(void* AItem1, void* AItem2)
{
	return AnsiCompareStr(
		static_cast<TProgramEntry*>(AItem1)->Name,
		static_cast<TProgramEntry*>(AItem2)->Name);
}
//---------------------------------------------------------------------------
void __fastcall TProgramModel::Initialize()
{
	try {
		FList = new TTypedObjectList<TProgramEntry>();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramModel::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
TProgramEntry* __fastcall TProgramModel::GetItem(int AIndex)
{
	return List->Items[AIndex];
}
//---------------------------------------------------------------------------
int __fastcall TProgramModel::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
void __fastcall TProgramModel::DoReload()
{
	if(OnReload != NULL) {
		OnReload(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TProgramModel::TProgramModel() :
	TObject(),
	FList(NULL),
	FOnReload(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TProgramModel::~TProgramModel()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TProgramModel::Organize()
{
	EnsureProgramsDirectoryExists();
	EnsureReadmeFileExists();
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
	LExecuteInfo.hwnd = Application->Handle;
	LExecuteInfo.lpVerb = "open";
	LExecuteInfo.lpFile = ProgramsDirectory.c_str();
	LExecuteInfo.nShow = SW_SHOWNORMAL;
	if(!::ShellExecuteEx(&LExecuteInfo)) {
		throw ECommandError("The program could not open the directory.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramModel::Reload()
{
	EnsureProgramsDirectoryExists();
	EnsureReadmeFileExists();
	{
		TAutoPointer<TTypedObjectList<TProgramEntry> > LList = new TTypedObjectList<TProgramEntry>();
		{
			TAutoPointer<TProgramEntry> LItem = new TProgramEntry();
			LList->Add(LItem);
			LItem.Release();
		}
		{
			AnsiString LDirectory = IncludeTrailingPathDelimiter(ProgramsDirectory);
			TAutoPointer<TFileEnumerator> LFileEnumerator = new TFileEnumerator(LDirectory + "*.lnk");
			while(LFileEnumerator->MoveNext()) {
				TAutoPointer<TProgramEntry> LItem = new TProgramEntry(LDirectory + LFileEnumerator->Current.cFileName);
				LList->Add(LItem);
				LItem.Release();
			}
		}
		LList->Sort(&ReloadCompare);
		FList = LList.Reset(FList);
	}
	DoReload();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool __fastcall TProgramSelections::GetItem(int AIndex)
{
	return AIndex == ComboBox->ItemIndex;
}
//---------------------------------------------------------------------------
int __fastcall TProgramSelections::GetCount()
{
	return ComboBox->Items->Count;
}
//---------------------------------------------------------------------------
int __fastcall TProgramSelections::GetSelectionCount()
{
	if(ComboBox->ItemIndex >= 0) {
		return 1;
	}
	else {
		return 0;
	}
}
//---------------------------------------------------------------------------
int __fastcall TProgramSelections::GetSelectionIndex()
{
	return ComboBox->ItemIndex;
}
//---------------------------------------------------------------------------
void __fastcall TProgramSelections::SetSelectionIndex(int ASelectionIndex)
{
	ComboBox->ItemIndex = ASelectionIndex;
}
//---------------------------------------------------------------------------
__fastcall TProgramSelections::TProgramSelections(TProgramModel* AProgramModel, TComboBox* AComboBox) :
	TObject(),
	FProgramModel(AProgramModel),
	FComboBox(AComboBox)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TProgramSelections::~TProgramSelections()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TProgramSelections::Reload()
{
	ComboBox->Items->BeginUpdate();
	try {
		ComboBox->Items->Clear();
		ComboBox->Items->Capacity = ProgramModel->Count;
		for(int LI = 0; LI < ProgramModel->Count; ++LI) {
			ComboBox->Items->Add(ProgramModel->Items[LI]->Name);
		}
		if(ComboBox->Items->Count > 0) {
			ComboBox->ItemIndex = 0;
		}
	}
	catch(...) {
		ComboBox->Items->EndUpdate();
		throw;
	}
	ComboBox->Items->EndUpdate();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TProgramCommands::TProgramCommands(TProgramModel* AProgramModel, TProgramSelections* AProgramSelections) :
	TCommands(),
	FProgramModel(AProgramModel),
	FProgramSelections(AProgramSelections)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TProgramCommands::~TProgramCommands()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TProgramCommands::Organize()
{
	if(!CanOrganize()) {
		HandleUnavailableFeature();
	}
	ProgramModel->Organize();
}
//---------------------------------------------------------------------------
bool __fastcall TProgramCommands::CanOrganize()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TProgramCommands::Reload()
{
	if(!CanReload()) {
		HandleUnavailableFeature();
	}
	ProgramModel->Reload();
}
//---------------------------------------------------------------------------
bool __fastcall TProgramCommands::CanReload()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TProgramCommands::Start()
{
	if(!CanStart()) {
		HandleUnavailableFeature();
	}
	ProgramModel->Items[ProgramSelections->SelectionIndex]->Start();
}
//---------------------------------------------------------------------------
bool __fastcall TProgramCommands::CanStart()
{
	return
		ProgramSelections->SelectionIndex >= 0 &&
		!ProgramModel->Items[ProgramSelections->SelectionIndex]->Default;
}
//---------------------------------------------------------------------------
void __fastcall TProgramCommands::Explore()
{
	if(!CanExplore()) {
		HandleUnavailableFeature();
	}
	ProgramModel->Items[ProgramSelections->SelectionIndex]->Explore();
}
//---------------------------------------------------------------------------
bool __fastcall TProgramCommands::CanExplore()
{
	return
		ProgramSelections->SelectionIndex >= 0 &&
		!ProgramModel->Items[ProgramSelections->SelectionIndex]->Default;
}
//---------------------------------------------------------------------------
void __fastcall TProgramCommands::ShowProperties()
{
	if(!CanShowProperties()) {
		HandleUnavailableFeature();
	}
	ProgramModel->Items[ProgramSelections->SelectionIndex]->ShowProperties();
}
//---------------------------------------------------------------------------
bool __fastcall TProgramCommands::CanShowProperties()
{
	return
		ProgramSelections->SelectionIndex >= 0 &&
		!ProgramModel->Items[ProgramSelections->SelectionIndex]->Default;
}
//---------------------------------------------------------------------------
void __fastcall TProgramCommands::LoadSettings(TMemIniFile* AValues, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".ProgramSelections";
	{
		int LSelectionIndex = AValues->ReadInteger(
			LSection,
			"SelectionIndex",
			ProgramSelections->SelectionIndex);
		if(LSelectionIndex >= 0 && LSelectionIndex < ProgramSelections->Count) {
			ProgramSelections->SelectionIndex = LSelectionIndex;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TProgramCommands::SaveSettings(TMemIniFile* AValues, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".ProgramSelections";
	{
		AValues->WriteInteger(
			LSection,
			"SelectionIndex",
			ProgramSelections->SelectionIndex);
	}
}
//---------------------------------------------------------------------------
} // End namespace App_programs
//---------------------------------------------------------------------------

