//---------------------------------------------------------------------------
#ifndef Moa_FilesH
#define Moa_FilesH
//---------------------------------------------------------------------------
// Version: 1.1.0
// Date: 2012-04-02 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moa_files {
//---------------------------------------------------------------------------
class EFileError : public Exception
{
public:
	__fastcall EFileError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TFileUtilities
{
private:
protected:
public:
	static bool __fastcall Exists(const AnsiString& APath);
	static AnsiString __fastcall GetShortPath(const AnsiString& APath);
	static AnsiString __fastcall GetLongPath(const AnsiString& APath);
	static DWORDLONG __fastcall GetSize(const AnsiString& APath);
	static DWORD __fastcall GetAttributes(const AnsiString& APath);
	static void __fastcall SetAttributes(const AnsiString& APath, DWORD AAttributes);
	static FILETIME __fastcall GetCreationTime(const AnsiString& APath);
	static void __fastcall SetCreationTime(const AnsiString& APath, const FILETIME& ATime);
	static FILETIME __fastcall GetLastWriteTime(const AnsiString& APath);
	static void __fastcall SetLastWriteTime(const AnsiString& APath, const FILETIME& ATime);
	static FILETIME __fastcall GetLastAccessTime(const AnsiString& APath);
	static void __fastcall SetLastAccessTime(const AnsiString& APath, const FILETIME& ATime);
	static void __fastcall GetTimes(const AnsiString& APath, FILETIME* ACreationTime = NULL, FILETIME* ALastWriteTime = NULL, FILETIME* ALastAccessTime = NULL);
	static void __fastcall SetTimes(const AnsiString& APath, const FILETIME* ACreationTime, const FILETIME* ALastWriteTime, const FILETIME* ALastAccessTime);
	static WIN32_FIND_DATA __fastcall GetData(const AnsiString& APath);
};
//---------------------------------------------------------------------------
class TFileDataUtilities
{
private:
protected:
public:
	static AnsiString __fastcall GetLongName(const WIN32_FIND_DATA& AData);
	static AnsiString __fastcall GetShortName(const WIN32_FIND_DATA& AData);
	static DWORDLONG __fastcall GetSize(const WIN32_FIND_DATA& AData);
	static FILETIME __fastcall GetCreationTime(const WIN32_FIND_DATA& AData);
	static FILETIME __fastcall GetLastWriteTime(const WIN32_FIND_DATA& AData);
	static FILETIME __fastcall GetLastAccessTime(const WIN32_FIND_DATA& AData);
	static DWORD __fastcall GetAttributes(const WIN32_FIND_DATA& AData);
};
//---------------------------------------------------------------------------
class TRegularFileUtilities
{
private:
protected:
public:
	static AnsiString __fastcall CreateTemporary();
	static AnsiString __fastcall CreateTemporary(const AnsiString& APrefix);
	static AnsiString __fastcall CreateTemporary(const AnsiString& APrefix, const AnsiString& ADirectory);
	static void __fastcall Move(const AnsiString& ASource, const AnsiString& ADestination);
	static void __fastcall Copy(const AnsiString& ASource, const AnsiString& ADestination, bool AOverwrite);
	static void __fastcall Delete(const AnsiString& APath);
	static bool __fastcall Exists(const AnsiString& APath);
};
//---------------------------------------------------------------------------
class TDirectoryUtilities
{
private:
protected:
public:
	static void __fastcall Create(const AnsiString& APath);
	static void __fastcall FullCreate(const AnsiString& APath);
	static void __fastcall Move(const AnsiString& ASource, const AnsiString& ADestination);
	static void __fastcall Delete(const AnsiString& APath);
	static bool __fastcall Exists(const AnsiString& APath);
};
//---------------------------------------------------------------------------
class TSpecialFiles
{
private:
protected:
public:
	static AnsiString __fastcall GetCurrentDirectory();
	static void __fastcall SetCurrentDirectory(const AnsiString& ADirectory);
	static AnsiString __fastcall GetSystemDirectory();
	static AnsiString __fastcall GetTemporaryDirectory();
	static AnsiString __fastcall GetWindowsDirectory();
};
//---------------------------------------------------------------------------
class EPathError : public EFileError
{
public:
	__fastcall EPathError(const AnsiString& AMessage) : EFileError(AMessage) {}
};
//---------------------------------------------------------------------------
class TPathUtilities
{
private:
protected:
public:
	static bool __fastcall IsAbsolute(const AnsiString& APath);
	static bool __fastcall IsRelative(const AnsiString& APath);
	static bool __fastcall IsPattern(const AnsiString& APath);
	static void __fastcall AssureAbsolute(const AnsiString& APath);
	static void __fastcall AssureRelative(const AnsiString& APath);
	static void __fastcall AssureNotPattern(const AnsiString& APath);
};
//---------------------------------------------------------------------------
class TFileEnumerator : public TObject
{
private:
	AnsiString FPattern;
	HANDLE FHandle;
	WIN32_FIND_DATA FData;
	bool __fastcall FindFirst();
	bool __fastcall FindNext();
	void __fastcall Finish();
	WIN32_FIND_DATA __fastcall GetCurrent();
	__property AnsiString Pattern = {read=FPattern};
	__property HANDLE Handle = {read=FHandle};
	__property WIN32_FIND_DATA Data = {read=FData};
protected:
public:
	__fastcall TFileEnumerator(const AnsiString& APattern);
	virtual __fastcall ~TFileEnumerator();
	virtual bool __fastcall MoveNext();
	virtual void __fastcall Reset();
	__property WIN32_FIND_DATA Current = {read=GetCurrent};
};
//---------------------------------------------------------------------------
} // End namespace Moa_files
using namespace Moa_files;
//---------------------------------------------------------------------------
#endif

