//---------------------------------------------------------------------------
// Version: 1.0.1
// Date: 2012-04-13 UTC+09:00
// Version: 1.0.0
// Date: 2012-03-31 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_AutoPointers.h"
#include "Moa_Tasks.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_tasks {
//---------------------------------------------------------------------------
class TTaskThread : public TThread
{
private:
	TNotifyEvent FOnExecute;
protected:
	virtual void __fastcall Execute();
	virtual void __fastcall DoExecute();
public:
	__fastcall TTaskThread(bool ACreatedSuspended);
	virtual __fastcall ~TTaskThread();
	using TThread::Synchronize;
	__property TNotifyEvent OnExecute = {read=FOnExecute, write=FOnExecute};
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TTaskThread::Execute()
{
	DoExecute();
}
//---------------------------------------------------------------------------
void __fastcall TTaskThread::DoExecute()
{
	if(OnExecute != NULL) {
		OnExecute(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TTaskThread::TTaskThread(bool ACreatedSuspended) :
	TThread(ACreatedSuspended),
	FOnExecute(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTaskThread::~TTaskThread()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TTask::SynchronizedDoExecute(void* AData)
{
	OnExecute(this);
}
//---------------------------------------------------------------------------
void __fastcall TTask::SynchronizedDoTerminate(void* AData)
{
	OnTerminate(this);
}
//---------------------------------------------------------------------------
void __fastcall TTask::SynchronizedDoProgress(void* AData)
{
	OnProgress(
		this,
		static_cast<int*>(AData)[0],
		static_cast<int*>(AData)[1]);
}
//---------------------------------------------------------------------------
void __fastcall TTask::SynchronizedDoStatus(void* AData)
{
	OnStatus(
		this,
		*static_cast<AnsiString*>(AData));
}
//---------------------------------------------------------------------------
void __fastcall TTask::SynchronizedDoException(void* AData)
{
	if(OnException != NULL) {
		OnException(
			this,
			static_cast<Exception*>(AData));
	}
	else {
		if(Application->OnException != NULL) {
			Application->OnException(Application, static_cast<Exception*>(AData));
		}
		else {
			Application->ShowException(static_cast<Exception*>(AData));
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::SynchronizedSynchronize()
{
	SynchronizeMethod(SynchronizeData);
}
//---------------------------------------------------------------------------
void __fastcall TTask::ThreadExecute(TObject* ASender)
{
	try {
		try {
			try {
				Execute();
			}
			catch(EAbort& LException) {
				// Do nothing
			}
			catch(Exception& LException) {
				throw;
			}
			catch(...) {
				throw Exception("An unexpected exception occured.");
			}
		}
		catch(Exception& LException) {
			DoException(&LException);
		}
	}
	catch(...) {
		// Do nothing
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::ThreadTerminate(TObject* ASender)
{
	try {
		try {
			DoTerminate();
		}
		__finally {
			FThread = NULL;
			if(FreeOnTerminate) {
				delete this;
			}
		}
	}
	catch(...) {
		// Do nothing
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::Execute()
{
	DoExecute();
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoExecute()
{
	if(OnExecute != NULL) {
		Synchronize(&SynchronizedDoExecute, NULL);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoTerminate()
{
	if(OnTerminate != NULL) {
		Synchronize(&SynchronizedDoTerminate, NULL);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoProgress(int ACurrent, int AAll)
{
	if(OnProgress != NULL) {
		int LArguments[2] = {ACurrent, AAll};
		Synchronize(&SynchronizedDoProgress, LArguments);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoStatus(const AnsiString& AStatus)
{
	if(OnStatus != NULL) {
		AnsiString LArgument = AStatus;
		Synchronize(&SynchronizedDoStatus, &LArgument);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoException(Exception* AException)
{
	Synchronize(&SynchronizedDoException, AException);
}
//---------------------------------------------------------------------------
__fastcall TTask::TTask() :
	TObject(),
	FThread(NULL),
	FStarted(false),
	FCanceled(false),
	FFreeOnTerminate(false),
	FSynchronizeMethod(NULL),
	FSynchronizeData(NULL),
	FOnExecute(NULL),
	FOnTerminate(NULL),
	FOnProgress(NULL),
	FOnStatus(NULL),
	FOnException(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTask::~TTask()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TTask::Perform()
{
	if(Started) {
		throw ETaskError("The program cannot perform the task.");
	}
	try {
		FStarted = true;
		TAutoPointer<TTaskThread> LThread = new TTaskThread(true);
		FThread = LThread;
		Thread->FreeOnTerminate = true;
		Thread->OnExecute = ThreadExecute;
		Thread->OnTerminate = ThreadTerminate;
		Thread->Resume();
		LThread.Release();
	}
	catch(...) {
		FThread = NULL;
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::Cancel()
{
	if(Canceled) {
		return;
	}
	FCanceled = true;
}
//---------------------------------------------------------------------------
void __fastcall TTask::Continue()
{
	if(Canceled) {
		throw ETaskCancel("The task was canceled.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::ReportProgress(int ACurrent, int AAll)
{
	DoProgress(ACurrent, AAll);
}
//---------------------------------------------------------------------------
void __fastcall TTask::ReportStatus(const AnsiString& AStatus)
{
	DoStatus(AStatus);
}
//---------------------------------------------------------------------------
void __fastcall TTask::ReportException(Exception* AException)
{
	DoException(AException);
}
//---------------------------------------------------------------------------
void __fastcall TTask::Synchronize(TTaskSynchronizeMethod AMethod, void* AData)
{
	if(::GetCurrentThreadId() == System::MainThreadID) {
		AMethod(AData);
		return;
	}
	FSynchronizeMethod = AMethod;
	FSynchronizeData = AData;
	Thread->Synchronize(&SynchronizedSynchronize);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TTaskMessageBox::SynchronizedExecute(void* AData)
{
	FResult = Application->MessageBox(
		FMessage,
		FTitle == NULL ? Application->Title.c_str() : FTitle,
		FFlags);
}
//---------------------------------------------------------------------------
__fastcall TTaskMessageBox::TTaskMessageBox(TTask* ATask, const char* AMessage, const char* ATitle, int AFlags) :
	FTask(ATask),
	FMessage(AMessage),
	FTitle(ATitle),
	FFlags(AFlags),
	FResult(0)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTaskMessageBox::~TTaskMessageBox()
{
	// Do nothing
}
//---------------------------------------------------------------------------
int __fastcall TTaskMessageBox::Execute()
{
	FTask->Synchronize(&SynchronizedExecute, NULL);
	return FResult;
}
//---------------------------------------------------------------------------
} // End namespace Moa_tasks
//---------------------------------------------------------------------------

